@@ -20,7 +20,6 @@
  * USA
  */
 
-#include <gnomescan.h>
 #include <math.h>
 #include "gnomescanui.h"
 #include "gnomescanui-intl.h"
@@ -30,12 +29,15 @@
 #define GET_PRIVATE(obj)				(G_TYPE_INSTANCE_GET_PRIVATE ((obj), GNOME_TYPE_SCAN_PREVIEW_AREA, GnomeScanPreviewAreaPrivate))
 #define	ANCHOR_SIDE					10
 #define debug_rect(r)					g_debug ("%s %i×%i+%i+%i", __FUNCTION__, r->width, r->height, r->x, r->y)
+#define debug_area(a)					g_debug ("%s %f×%f+%f+%f", __FUNCTION__, a->width, a->height, a->x, a->y)
 
 typedef struct _GnomeScanPreviewAreaPrivate		GnomeScanPreviewAreaPrivate;
 
 struct _GnomeScanPreviewAreaPrivate {
   gboolean		dispose_has_run;
 
+  GtkTooltips		*tooltips;
+
   gboolean		button_pressed;
   GdkPixbuf		*image;
   /* The rotation degree to apply to the current preview */
@@ -45,11 +47,13 @@ struct _GnomeScanPreviewAreaPrivate {
 
   cairo_t		*cr;
   cairo_surface_t	*buffer;
+  gboolean		buffer_uptodate;
 
   /* origine of selection */
   GdkPoint		*orig;
   /* region of interest */
   GdkRectangle		*roi;
+  gboolean		custom;
   /* current pointer coord */
   GdkPoint		*coord;
   /* current over flyed anchor */
@@ -107,10 +111,16 @@ void				gspa_rotate_counter_clockwise_clicked		(GnomeScanPreviewArea *pa,
 void				gspa_rotate_clockwise_clicked			(GnomeScanPreviewArea *pa,
 										 gpointer data);
 
+void				gspa_context_changed				(GnomeScanContext *context,
+										 gchar* option_name,
+										 GnomeScanPreviewArea *gspa);
+
 /********************************
  * 	      OTHER		*
  ********************************/
 
+void				gspa_deprecate_buffer				(GnomeScanPreviewArea *pa);
+
 void				gspa_draw_buffer				(GnomeScanPreviewArea *pa);
 
 GdkRectangle*			gspa_compute_anchor_rect			(GtkAnchorType anchor,
@@ -187,10 +197,13 @@ gnome_scan_preview_area_init (GnomeScanPreviewArea *widget)
   widget->drawing_area = NULL;
 
   priv->dispose_has_run = FALSE;
+  priv->tooltips	= NULL;
   priv->button_pressed	= FALSE;
   priv->buffer		= NULL;
+  priv->buffer_uptodate	= FALSE;
   priv->orig		= g_new0 (GdkPoint, 1);
   priv->roi		= g_new0 (GdkRectangle, 1);
+  priv->custom		= FALSE;
   priv->coord		= g_new0 (GdkPoint, 1);
   priv->rotate		= GDK_PIXBUF_ROTATE_NONE;
   priv->rotation	= GDK_PIXBUF_ROTATE_NONE;
@@ -211,6 +224,7 @@ gnome_scan_preview_area_dispose (GObject *obj)
   /* unref devices */
   g_object_unref (widget->context);
   g_object_unref (widget->drawing_area);
+  g_object_unref (priv->tooltips);
   g_free (priv->orig);
   priv->dispose_has_run = TRUE;
 
@@ -296,8 +310,10 @@ gnome_scan_preview_area_new (GnomeScanContext *context)
 			 GDK_POINTER_MOTION_HINT_MASK |
 			 GDK_SCROLL_MASK);
 
-  /*   gtk_widget_add_events (da, */
-  /* 			 GDK_ALL_EVENTS_MASK); */
+/*   priv->tooltips = gtk_tooltips_new (); */
+/*   gtk_tooltips_set_tip (GTK_TOOLTIPS (priv->tooltips), GTK_WIDGET (gspa), */
+/* 			_("Preview area."), */
+/* 			_("Use the preview to select the scan area and the rotation to get the right image.")); */
 
   g_signal_connect (context,
 		    "scanner-selected",
@@ -334,6 +350,11 @@ gnome_scan_preview_area_new (GnomeScanContext *context)
 		    G_CALLBACK (gspa_scrolled),
 		    gspa);
 
+  g_signal_connect (context,
+		    "changed",
+		    G_CALLBACK (gspa_context_changed),
+		    gspa);
+
   return GTK_WIDGET (gspa);
 }
 
@@ -353,6 +374,68 @@ gnome_scan_preview_area_refresh (GnomeScanPreviewArea *area)
   gnome_scan_context_acquire_preview (area->context);
 }
 
+/**
+ * gnome_scan_preview_area_button_refresh:
+ * @area: 
+ * 
+ * 
+ * 
+ * Returns: 
+ **/
+GtkWidget*
+gnome_scan_preview_area_button_refresh (GnomeScanPreviewArea *area)
+{
+  GtkWidget *button;
+
+  button = gtk_button_new_from_stock (GTK_STOCK_REFRESH);
+  g_signal_connect_swapped (button,
+			    "clicked",
+			    (GCallback) gnome_scan_preview_area_refresh,
+			    area);
+
+  return button;
+}
+
+/**
+ * gnome_scan_preview_area_button_rotate_cw:
+ * @area: 
+ * 
+ * 
+ * 
+ * Returns: 
+ **/
+GtkWidget*
+gnome_scan_preview_area_button_rotate_cw (GnomeScanPreviewArea *area)
+{
+  GtkWidget *button;
+  button = gtk_button_new_from_stock (GS_STOCK_ROTATE_CLOCKWISE);
+  g_signal_connect_swapped (button,
+			    "clicked",
+			    (GCallback) GNOME_SCAN_PREVIEW_AREA_GET_CLASS (area)->rotate_clockwise_clicked,
+			    area);
+  return button;
+}
+
+/**
+ * gnome_scan_preview_area_button_rotate_ccw:
+ * @area: 
+ * 
+ * 
+ * 
+ * Returns: 
+ **/
+GtkWidget*
+gnome_scan_preview_area_button_rotate_ccw (GnomeScanPreviewArea *area)
+{
+  GtkWidget *button;
+  button = gtk_button_new_from_stock (GS_STOCK_ROTATE_COUNTER_CLOCKWISE);
+  g_signal_connect_swapped (button,
+			    "clicked",
+			    (GCallback) GNOME_SCAN_PREVIEW_AREA_GET_CLASS (area)->rotate_counter_clockwise_clicked,
+			    area);
+  return button;
+}
+
 /********************************
  * 	     CALLBACKS		*
  ********************************/
@@ -366,9 +449,9 @@ gspa_scanner_selected (GnomeScanContext *context,
   GnomeScanPreviewArea *pa;
   GnomeScanGeometry *dim;
   GnomeScanArea *a;
-  GnomeScanResult *info;
-  GnomeScannerSource source;
+  GnomeScanResult *result;
   GdkPixmap *pixmap;
+  GdkPixbuf *icon;
   gint width, height, size;
   cairo_t *cr;
 
@@ -380,49 +463,43 @@ gspa_scanner_selected (GnomeScanContext *context,
    * create a pixbuf with the device size.
    */
 
-  source = gnome_scan_context_get_source (context);
+  /* simulate acquisition of a 50dpi preview */
+  result = g_new0 (GnomeScanResult, 1);
+  result->xresolution = 50.;
+  result->yresolution = 50.;
+  result->source = gnome_scan_context_get_source (context);
 
-  if (source == GNOME_SCANNER_SOURCE_FLATBED) {
-    gnome_scan_context_acquire_preview (context);
+  /* create an empty pixbuf at the correct geometry */
+  dim = gnome_scanner_get_geometry (scanner);
+  width = gnome_scan_pixel_from_mm (dim->width, 50);
+  height = gnome_scan_pixel_from_mm (dim->height, 50);
+  pixmap = gdk_pixmap_new (NULL, width, height, 24);
+
+  /* Draw a big (and nice) logo */
+  cr = gdk_cairo_create (GDK_DRAWABLE (pixmap));
+  cairo_set_source_rgb (cr, 1., 1., 1.);
+  cairo_paint (cr);
+
+  /* icon take 70% of the picture */	
+  size = MIN (width, height) * .6;
+  if (icon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+				       "document",
+				       size,
+				       GTK_ICON_LOOKUP_FORCE_SVG,
+				       NULL)) {
+    /* we put the file as close as possible to the border. Often,
+       icons have a large empty border around image */
+    gdk_cairo_set_source_pixbuf (cr, icon, - (size * .1), - (size * .05));
   }
   else {
-    /* simulate acquisition of a 50dpi preview */
-    info = g_new0 (GnomeScanResult, 1);
-    info->resolution = 50;
-    info->source = gnome_scan_context_get_source (context);
+    cairo_set_source_rgb (cr, .8, .8, .8);
+  }
 
-    /* create an empty pixbuf at the correct geometry */
-    dim = gnome_scanner_get_geometry (scanner);
-    width = gnome_scan_pixel_from_mm (dim->width, 50);
-    height = gnome_scan_pixel_from_mm (dim->height, 50);
+  cairo_paint (cr);
+  cairo_destroy (cr);
 
-    pixmap = gdk_pixmap_new (NULL,
-			     width,
-			     height,
-			     24);
-			   
-    /* Draw a big (and nice) logo */
-
-    cr = gdk_cairo_create (GDK_DRAWABLE (pixmap));
-    cairo_set_source_rgb (cr, 1., 1., 1.);
-    cairo_paint (cr);
-
-    /* icon take 70% of the picture */
-    size = MIN (width, height) * .7;
-
-    gdk_cairo_set_source_pixbuf (cr,
-				 gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
-							   PACKAGE,
-							   size,
-							   GTK_ICON_LOOKUP_FORCE_SVG,
-							   NULL),
-				 (width - size) / 2,
-				 (height - size) / 2);
-    cairo_paint (cr);
-    cairo_destroy (cr);
-
-    /* create pixbuf */
-    info->image  = gdk_pixbuf_get_from_drawable (NULL,
+  /* create pixbuf */
+  result->image  = gdk_pixbuf_get_from_drawable (NULL,
 						 GDK_DRAWABLE (pixmap),
 						 gdk_screen_get_default_colormap (gdk_screen_get_default ()),	
 						 0, 0,
@@ -430,17 +507,14 @@ gspa_scanner_selected (GnomeScanContext *context,
 						 width,
 						 height);
 
-    /* simulate the acquisition of the pixbuf. Not very clean. */
-    gspa_preview_received (context,
-			   info,
-			   area);
-  }
+  /* simulate the acquisition of the pixbuf. Not very clean. */
+  gspa_preview_received (context, result, area);
 }
 
 /* store pixbuf and query draw */
 void
 gspa_preview_received (GnomeScanContext *context,
-		       GnomeScanResult *info,
+		       GnomeScanResult *result,
 		       GtkWidget *area)
 { 
   GnomeScanPreviewArea *pa = GNOME_SCAN_PREVIEW_AREA (area);
@@ -452,9 +526,10 @@ gspa_preview_received (GnomeScanContext *context,
     g_object_unref (priv->image);
   }
   /* ref new one. */
-  priv->image = info->image;
+  priv->image = result->image;
   g_object_ref (priv->image);
-  pa->resolution = info->resolution;
+  /* preview are always in 50x50 dpi */
+  pa->resolution = result->xresolution;
 
   /*
    * We received a 0° rotated image.
@@ -487,6 +562,31 @@ gspa_preview_received (GnomeScanContext *context,
   gspa_apply_transformations (pa);
 }
 
+void
+gspa_context_changed (GnomeScanContext *context,
+		      gchar* option_name,
+		      GnomeScanPreviewArea *gspa)
+{
+  GnomeScanPreviewAreaPrivate *priv;
+  GnomeScanArea *area;
+  GdkRectangle *rect;
+
+  if (!g_ascii_strcasecmp (option_name, "area")) {
+    priv = GET_PRIVATE (gspa);
+    area = gnome_scan_context_get_area (context);
+    priv->custom = area->custom;
+    priv->roi->x = gnome_scan_pixel_from_mm (area->x, gspa->resolution);
+    priv->roi->y = gnome_scan_pixel_from_mm (area->y, gspa->resolution);
+    priv->roi->width = gnome_scan_pixel_from_mm (area->width, gspa->resolution);
+    priv->roi->height = gnome_scan_pixel_from_mm (area->height, gspa->resolution);
+
+    gspa_gdk_rectangle_rotate_simple (priv->roi, priv->rotation,
+				      gdk_pixbuf_get_width (priv->image),
+				      gdk_pixbuf_get_height (priv->image));
+    gspa_deprecate_buffer (gspa);
+  }
+}
+
 /* paint buffer in GdkWindow */
 gboolean
 gspa_exposed (GtkWidget *da,
@@ -498,8 +598,8 @@ gspa_exposed (GtkWidget *da,
   gint lw, lh;
   cairo_t *cr;
 
-  if (!priv->buffer) {
-    return FALSE;
+  if (!priv->buffer_uptodate) {
+    gspa_draw_buffer (gspa);
   }
 
   cr = gdk_cairo_create (gspa->drawing_area->window);
@@ -521,20 +621,22 @@ gspa_button_pressed (GtkWidget *widget,
   GnomeScanPreviewArea *pa = GNOME_SCAN_PREVIEW_AREA (area);
   GnomeScanPreviewAreaPrivate *priv = GET_PRIVATE (pa);
 
-  priv->button_pressed = TRUE;
+  if (priv->custom || priv->anchor == GTK_ANCHOR_CENTER) {
+    priv->button_pressed = TRUE;
 
-  priv->orig->x = event->x;
-  priv->orig->y = event->y;
+    priv->orig->x = event->x;
+    priv->orig->y = event->y;
 
-  gdk_pointer_grab (pa->drawing_area->window,
-		    TRUE,
-		    GDK_BUTTON_RELEASE_MASK |
-		    GDK_BUTTON1_MOTION_MASK |
-		    GDK_POINTER_MOTION_MASK |
-		    GDK_POINTER_MOTION_HINT_MASK,
-		    pa->drawing_area->window,
-		    NULL,
-		    event->time);
+    gdk_pointer_grab (pa->drawing_area->window,
+		      TRUE,
+		      GDK_BUTTON_RELEASE_MASK |
+		      GDK_BUTTON1_MOTION_MASK |
+		      GDK_POINTER_MOTION_MASK |
+		      GDK_POINTER_MOTION_HINT_MASK,
+		      pa->drawing_area->window,
+		      NULL,
+		      event->time);
+  }
 
   return FALSE;
 }
@@ -603,8 +705,7 @@ gspa_mouse_moved (GtkWidget *widget,
       break;
     }
 
-    gspa_draw_buffer (pa);
-    gtk_widget_queue_draw (pa->drawing_area);
+    gspa_deprecate_buffer (pa);
 
     /* TODO: auto scroll on edge */
 
@@ -639,9 +740,7 @@ gspa_button_released (GtkWidget *widget,
   GnomeScanPreviewAreaPrivate *priv = GET_PRIVATE (area);
 
   priv->button_pressed = FALSE;
-
   gdk_pointer_ungrab (event->time);
-
   gspa_send_area (GNOME_SCAN_PREVIEW_AREA (area));
 
   return FALSE;
@@ -707,10 +806,23 @@ gspa_rotate_clockwise_clicked (GnomeScanPreviewArea *pa,
   gspa_apply_transformations (pa);
 }
 
+
 /********************************
  * 	      OTHER		*
  ********************************/
 
+/*
+ * ask for buffer update and queue widget for drawing.
+ */
+void
+gspa_deprecate_buffer (GnomeScanPreviewArea *pa)
+{
+  GnomeScanPreviewAreaPrivate *priv = GET_PRIVATE (pa);
+  priv->buffer_uptodate = FALSE;
+  gtk_widget_queue_draw (pa->drawing_area);
+}
+
+
 /* 
  * Draw the pixbuf priv->image inside priv->buffer
  * cairo_surface_t. Also draw faded white rectangles outside
@@ -767,7 +879,12 @@ gspa_draw_buffer (GnomeScanPreviewArea *pa)
   cairo_rectangle (cr, 0, 0, width, height);
   gdk_cairo_rectangle (cr, a);
   cairo_clip_preserve (cr);
-  gs_enum_foreach (GTK_TYPE_ANCHOR_TYPE, (GSEFunc) gspa_draw_anchor_path, pa);
+  if (priv->custom) {
+    gs_enum_foreach (GTK_TYPE_ANCHOR_TYPE, (GSEFunc) gspa_draw_anchor_path, pa);
+  }
+  else {
+    gspa_draw_anchor_path (GTK_ANCHOR_CENTER, pa);
+  }
   cairo_close_path (cr);
   cairo_clip (cr);
 
@@ -778,7 +895,12 @@ gspa_draw_buffer (GnomeScanPreviewArea *pa)
   /* draw anchors */
   cairo_reset_clip (cr);
   cairo_new_path (cr);
-  gs_enum_foreach (GTK_TYPE_ANCHOR_TYPE, (GSEFunc) gspa_draw_anchor_path, pa);
+  if (priv->custom) {
+    gs_enum_foreach (GTK_TYPE_ANCHOR_TYPE, (GSEFunc) gspa_draw_anchor_path, pa);
+  }
+  else {
+    gspa_draw_anchor_path (GTK_ANCHOR_CENTER, pa);
+  }
 
   /* fill a 50% white */
   cairo_set_source_rgba (cr, 1., 1., 1., .5);
@@ -802,6 +924,8 @@ gspa_draw_buffer (GnomeScanPreviewArea *pa)
   /* Free all */
   cairo_destroy (cr);
   g_free (a);
+  priv->buffer_uptodate = TRUE;
+  gtk_widget_queue_draw (pa->drawing_area);
 }
 
 /*
@@ -916,7 +1040,12 @@ void
 gspa_anchor_over (GnomeScanPreviewArea *pa)
 {
   GET_PRIVATE (pa)->anchor = -1;
-  gs_enum_foreach (GTK_TYPE_ANCHOR_TYPE, (GSEFunc) gspa_in_anchor, pa);
+  if (GET_PRIVATE (pa)->custom) {
+    gs_enum_foreach (GTK_TYPE_ANCHOR_TYPE, (GSEFunc) gspa_in_anchor, pa);
+  }
+  else {
+    gspa_in_anchor (GTK_ANCHOR_CENTER, pa);
+  }
 }
 
 /*
@@ -982,13 +1111,15 @@ gspa_send_area (GnomeScanPreviewArea *pa)
 				    360 - priv->rotation,
 				    gdk_pixbuf_get_width (priv->image),
 				    gdk_pixbuf_get_height (priv->image));
+  /*   debug_rect (roi); */
 
-#define set_mm(var,px,res)	var = gnome_scan_mm_from_pixel (px, res)
+  area->custom = priv->custom;
+  area->x = gnome_scan_mm_from_pixel (roi->x, pa->resolution);
+  area->y = gnome_scan_mm_from_pixel (roi->y, pa->resolution);
+  area->width = gnome_scan_mm_from_pixel (roi->width, pa->resolution);
+  area->height = gnome_scan_mm_from_pixel (roi->height, pa->resolution);
 
-  set_mm (area->x, roi->x, pa->resolution);
-  set_mm (area->y, roi->y, pa->resolution);
-  set_mm (area->width, roi->width, pa->resolution);
-  set_mm (area->height, roi->height, pa->resolution);
+  /*   debug_area (area); */
 
   gnome_scan_context_set_area (pa->context, area);
 }
@@ -1039,9 +1170,8 @@ gspa_apply_transformations (GnomeScanPreviewArea *pa)
   set_if_not_included(priv->roi->width, width - priv->roi->x, 0, width - priv->roi->x); 
   set_if_not_included(priv->roi->height, height - priv->roi->y, 0, height - priv->roi->y);
 
+  gspa_deprecate_buffer (pa);
   gspa_send_area (pa);
-  gspa_draw_buffer (pa);
-  gtk_widget_queue_draw (pa->drawing_area);
 }
 
 /*
